/*   
    (C) 2009 Vadim Peretokin
    
    (C) 2007, 2008 Gianni Moschini

    This file is part of GiftWrap.

    GiftWrap is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    GiftWrap is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GiftWrap.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <libgnomeui/libgnomeui.h>
#include "def.h"
#include "callback.h"
#include "done.h"

gchar *gettext_fromtextview_list(struct _allwidgets *widg)
{
    gchar *text = NULL;

    GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widg->textview_listfiles));

    GtkTextIter start, end;

    if (buffer) {
        gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &start);

        gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(buffer), &end, 1);

        text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &start, &end, FALSE);

        if (text)
            text = g_strstrip(text);
    }

    return text;

}

gboolean update_textview_filelist(gpointer data)
{
    struct _allwidgets *widg = data;

    if (widg->output) {
        set_textview_text(widg->textview_listfiles, widg->output);

        gtk_widget_show(widg->scrolled_listfiles);
        gtk_widget_hide(widg->progress_list);

        g_free(widg->output);
        widg->output = NULL;
    }

    return FALSE;
}

gpointer getoutput_from_listing(gpointer data)
{
    struct _allwidgets *widg = data;

    gchar *output = NULL;

    if (widg->cmd) {
        if (widg->debug)
            g_print("Executing: %s\n", widg->cmd);

        g_spawn_command_line_sync(widg->cmd, &output, NULL, NULL, NULL);

        if (widg->workingdir) {
            g_free(widg->workingdir);
            widg->workingdir = NULL;
        }

        g_free(widg->cmd);
        widg->cmd = NULL;

    }

    if (output) {
        widg->output = output;

        g_idle_add(update_textview_filelist, data);
    }

    g_idle_add(clear_timer, data);

    return NULL;
}

gpointer action_do(gpointer data)
{
    struct _allwidgets *widg = data;

    if (widg->cmd) {
        if (widg->debug)
            g_print("Executing: %s\n", widg->cmd);

        system(widg->cmd);

        if (widg->workingdir) {
            g_free(widg->workingdir);
            widg->workingdir = NULL;
        }

        g_free(widg->cmd);
        widg->cmd = NULL;

        if (widg->aftercmd)
            g_idle_add(widg->aftercmd, data);
    }

    return NULL;
}

gchar *get_extract_command(struct _allwidgets * widg, gchar * from, const gchar * to)
{
    gchar *cmd = NULL;

    if (g_file_test(to, G_FILE_TEST_EXISTS)) {
        gchar *dirname = g_path_get_dirname(to);

        if (dirname) {
            cmd = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widg->file_chooser));

            if (cmd != NULL) {
                gchar *archive = g_path_get_basename(cmd);

                g_free(cmd);
                cmd = NULL;

                if (archive != NULL) {

                    cmd = g_strconcat("cp ", from, " ", dirname,
                                      " && cd ", dirname, " && tar -xf ", archive, NULL);

                    g_free(archive);
                }
            }
            g_free(dirname);
        }
    }

    return cmd;
}

gchar *copy_from_to(struct _allwidgets * widg, gchar * from, const gchar * to)
{
    gchar *cmd = NULL;

    gboolean exist = g_file_test(to, G_FILE_TEST_EXISTS);

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder))) {
        /* folder */

        if (exist) {
            if (widg->debug)
                g_printf("destination folder exists\n");

            return NULL;
        }

        cmd = g_strconcat("cp -r ", from, " ", to, NULL);
    } else {
        /* archive */
        if (!exist) {
            if (widg->debug)
                g_print("mkdir %s\n", to);

            g_mkdir(to, S_IRWXU | S_IRWXG | S_IRWXG);
        }

        cmd = get_extract_command(widg, from, to);
    }

    return cmd;
}
