/*   
    (C) 2009 Vadim Peretokin
    
    (C) 2007, 2008 Gianni Moschini

    This file is part of GiftWrap.

    GiftWrap is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    GiftWrap is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GiftWrap.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <libgen.h>
#include <libgnomeui/libgnomeui.h>
#include <glib/gprintf.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include "def.h"
#include "callback.h"
#include "done.h"
#include "action.h"
#include "config.h"

#define FILELISTING_PROCESS_RUNNING "There is a process running that is scanning the folder listing. Please check the Files listing tab and wait that it finishes before continuing."

void set_textview_text(GtkWidget * textview, gchar * text)
{
    GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));

    if (text != NULL)
        gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), text, -1);
    else {
        GtkTextIter start, end;

        gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &start);

        gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer), &end);

        gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &start, &end);

    }
}

gboolean clear_hint(GtkWidget * widget, GdkEventWindowState * event, gpointer data)
{
    struct _allwidgets *widg = data;
    if (gtk_window_get_urgency_hint(GTK_WINDOW(widg->window)))
        gtk_window_set_urgency_hint(GTK_WINDOW(widg->window), FALSE);

    return FALSE;
}

void start_nautilus(struct _allwidgets *widg)
{
    gchar *cmd;

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->check_end))) {
        gchar *dirname = g_path_get_dirname(widg->cmd);

        if (dirname) {
            cmd = g_strconcat("nautilus file://", dirname, NULL);

            g_free(dirname);
        }

        if (cmd) {
            if (widg->debug)
                g_print("%s\n", cmd);

            g_spawn_command_line_async(cmd, NULL);
            g_free(cmd);
        }
    }

}

void clear_quit(struct _allwidgets *widg)
{
    if (widg->timer != 0) {
        g_source_remove(widg->timer);
        widg->timer = 0;
    }

    if (widg->cmd) {
        start_nautilus(widg);

        g_free(widg->cmd);
        widg->cmd = NULL;
    }

    if (widg->mimefilters) {
        GSList *tmp;

        for (tmp = widg->mimefilters; tmp; tmp = tmp->next) {
            g_free(tmp->data);
            tmp->data = NULL;
        }

        g_slist_free(widg->mimefilters);
        widg->mimefilters = NULL;
    }

    if (widg->operation_list) {
        g_list_free(widg->operation_list);
        widg->operation_list = NULL;
    }

    gtk_main_quit();

}

void finish(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = (struct _allwidgets *) user_data;

    clear_quit(widg);
}

gboolean cancel(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = (struct _allwidgets *) user_data;

    clear_quit(widg);

    return TRUE;
}

gboolean quit(GtkWidget * window, GdkEvent * event, gpointer data)
{
    struct _allwidgets *widg = data;

    clear_quit(widg);

    return TRUE;
}

void error_dialog(GtkDialog * dialog, gint rid, gpointer data)
{
    struct _allwidgets *widg = data;

    gtk_widget_destroy(GTK_WIDGET(dialog));

    gtk_widget_destroy(widg->window);
}

void close_dialog(GtkDialog * dialog, gint rid, gpointer data)
{

    gtk_widget_destroy(GTK_WIDGET(dialog));

}

gboolean update_status(gpointer data)
{
    GtkWidget *progressbar = data;

    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progressbar));

    return TRUE;
}

int delete_projectdir(struct _allwidgets *widg, const gchar * projectdir)
{
    int retval = -1;

    gchar *cmd = g_strconcat("rm -r ", projectdir, NULL);

    if (cmd) {
        gchar *output = NULL, *error = NULL;

        if (widg->debug)
            g_print("%s\n", cmd);

        retval = system(cmd);

        g_free(cmd);

        if (output)
            g_free(output);

        if (error)
            g_free(error);

    }

    return retval;
}

void create_tmpdir(struct _allwidgets *widg, const gchar * tmpdir)
{
    if (!g_file_test(tmpdir, G_FILE_TEST_EXISTS)) {
        if (widg->debug)
            g_print("mkdir %s\n", tmpdir);

        g_mkdir(tmpdir, S_IRWXU | S_IRWXG | S_IRWXG);
    }
}

void set_dateversion(struct _allwidgets *widg)
{
    gchar *buf = NULL;

    g_spawn_command_line_sync("date +%Y%m%d", &buf, NULL, NULL, NULL);

    if (buf) {
        if (strlen(buf) > 1)
            buf[strlen(buf) - 1] = '\0';

        if (widg->debug)
            g_print("date: %s\n", buf);

        gchar *text = g_strconcat("0.0.", buf, NULL);

        g_free(buf);

        if (text) {
            gtk_entry_set_text(GTK_ENTRY(widg->entry_version), text);
            g_free(text);
        }
    } else
        gtk_entry_set_text(GTK_ENTRY(widg->entry_version), "0.0.1");
}

void set_product_nameversion(struct _allwidgets *widg, const gchar * product)
{
    gchar *name = NULL;
    gchar *version = NULL;
    gchar *lower = NULL;
    gchar *extension = NULL;
    gchar *tmp = NULL;

    gboolean set = FALSE;

    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder))) {
        if (g_str_has_suffix(product, ".tar.gz")
            || g_str_has_suffix(product, ".tar.bz2"))

            extension = g_strrstr(product, ".tar");

        else
            extension = g_strrstr(product, ".");
    }

    if (extension) {
        glong sizeprod = g_utf8_strlen(product, -1)
            - g_utf8_strlen(extension, -1);

        tmp = (gchar *) g_malloc(sizeof(gchar) * sizeprod + 1);

        if (tmp) {
            if (sizeprod > 1)
                g_utf8_strncpy(tmp, product, sizeprod);
        }
    }

    if (!tmp)
        tmp = g_strdup(product);

    if (tmp) {
        version = g_strrstr(tmp, "-");

        if (version && strlen(version) > 0) {
            name = g_strndup(tmp, strlen(tmp) - strlen(version));

            set = TRUE;
            gtk_entry_set_text(GTK_ENTRY(widg->entry_version), version + 1);
        }

        if (!name)
            name = g_strdup(tmp);

        g_free(tmp);

    }

    if (!set)
        set_dateversion(widg);

    if (name) {
        lower = g_ascii_strdown(name, -1);

        if (lower) {
            gtk_entry_set_text(GTK_ENTRY(widg->entry_product), lower);
            gtk_entry_set_text(GTK_ENTRY(widg->entry_deppkgname), lower);
            g_free(lower);
        } else {
            gtk_entry_set_text(GTK_ENTRY(widg->entry_product), name);
            gtk_entry_set_text(GTK_ENTRY(widg->entry_deppkgname), name);
        }

        g_free(name);
    }

}

void show_error_dialog(struct _allwidgets *widg,
                       void (*callback) (GtkDialog *, gint, gpointer), const gchar * message)
{
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new(GTK_WINDOW(widg->window),
                                    GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, message);

    g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(callback), widg);

    gtk_widget_show_all(dialog);
}

void get_software_version(struct _allwidgets *widg)
{
    gchar *dir = NULL;

    /*if(g_file_test(tmpdir, G_FILE_TEST_IS_DIR) ) */
    {
        dir = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widg->file_chooser));

        if (dir) {
/*			gchar *dirname = g_path_get_dirname(tmpdir);
		
			if(dirname)*/
            {
                gchar *product = g_path_get_basename(dir);

                if (product) {
                    /*if(g_ascii_strcasecmp(dir, dirname)  != 0) */
                    set_product_nameversion(widg, product);

                    g_free(product);
                }

                /*g_free(dirname); */
            }

            g_free(dir);
        }

    }
    /*else
       show_error_dialog(widg, tmpdir, error_dialog); */

}

/*
 * Go to the second page and detect product version
 */
void ask_productmaintainer(struct _allwidgets *widg)
{
    const gchar *envstring = g_getenv("DEBFULLNAME");
    if (envstring)
        gtk_entry_set_text(GTK_ENTRY(widg->entry_name), envstring);

    envstring = g_getenv("DEBEMAIL");
    if (envstring)
        gtk_entry_set_text(GTK_ENTRY(widg->entry_email), envstring);

}

void select_currentdir_toggle(GtkToggleButton * togglebutton, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder))) {
        gtk_file_chooser_set_action(GTK_FILE_CHOOSER(widg->file_chooser),
                                    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_select_folder),
                                            "Select the application source folder");
    } else {
        gtk_file_chooser_set_action(GTK_FILE_CHOOSER(widg->file_chooser), GTK_FILE_CHOOSER_ACTION_OPEN);
        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_select_folder),
                                            "Select the application archive");
    }
}

void check_selection(struct _allwidgets *widg)
{
    gboolean is_a_source_folder, source_folder_checkbox;
    const gchar *mime = NULL;
    gchar *cmd = NULL;
    gchar *file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widg->file_chooser));

    /* Is the 'source folder' checkbox toggled? */
    source_folder_checkbox = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder));

    if (file != NULL) {
        /* Use 'find' on directories, and atool on archives */
        if (g_file_test(file, G_FILE_TEST_IS_DIR)) {
            cmd = g_strconcat("find ", file, NULL);
            is_a_source_folder = TRUE;
        } else
            cmd = g_strconcat("atool --list --quiet ", file, NULL);

        gtk_label_set_text(GTK_LABEL(widg->label_tmpdirectory), file);

        g_free(file);
    } else
        is_a_source_folder = TRUE;

    if (cmd) {

        gtk_widget_hide(widg->scrolled_listfiles);
        gtk_widget_show(widg->progress_list);

        if (widg->debug)
            g_printf("Listing files:\n%s\n", cmd);

        widg->timer = g_timeout_add(100, update_status, widg->progress_list);

        if (widg->cmd)
            g_free(widg->cmd);
        widg->cmd = cmd;

        g_thread_create(getoutput_from_listing, widg, FALSE, NULL);

    }

    if (is_a_source_folder == FALSE) {
        gchar *fileuri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(widg->file_chooser));

        GnomeVFSURI *uri = NULL;

        if (fileuri) {
            uri = gnome_vfs_uri_new(fileuri);
            g_free(fileuri);

        }

        if (uri) {

            mime = gnome_vfs_get_mime_type_common(uri);
            g_free(uri);
        }
    }

    if (mime != NULL) {
        GSList *tmp;
        gboolean valid = FALSE;

        for (tmp = widg->mimefilters; tmp;) {
            if (g_ascii_strcasecmp((gchar *) tmp->data, mime) == 0) {
                valid = TRUE;
                tmp = NULL;
            } else
                tmp = tmp->next;
        }

        if (valid == FALSE) {
            show_error_dialog(widg, close_dialog, "File format not supported.");
            return;
        }
    }

    //~ if (is_a_source_folder == source_folder_checkbox) {

        ask_productmaintainer(widg);

        get_software_version(widg);

        /*gchar *tmpdir = g_strconcat(
           g_getenv("HOME"), "/.giftwrap/",
           gtk_entry_get_text(GTK_ENTRY(widg->entry_product)), "/", 
           gtk_entry_get_text(GTK_ENTRY(widg->entry_product)), "-", 
           gtk_entry_get_text(GTK_ENTRY(widg->entry_version)), NULL);

           if(tmpdir)
           {
           gtk_file_chooser_set_current_folder(
           GTK_FILE_CHOOSER(widg->filechooserbutton_tmpdirectory),
           tmpdir);
           g_free(tmpdir);
           } */

        gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_space));

    //~ }

}

gboolean check_selected_file(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    check_selection(widg);

    return TRUE;
}

void fileselection(GtkFileChooser * filechooser, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    check_selection(widg);
}

gboolean switchto_pageready(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_ready));

    return TRUE;
}

gboolean start_build_page(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_doing));

    gnome_druid_set_buttons_sensitive(GNOME_DRUID(widg->druid), FALSE, FALSE, TRUE, FALSE);

    build_package(widg, "fakeroot");

    return TRUE;
}

gboolean validate_version(struct _allwidgets * widg)
{
    gboolean ok = FALSE;

    gchar *textinlist = gettext_fromtextview_list(widg);

    if (textinlist) {
        gchar *direntry = g_strconcat(gtk_entry_get_text(GTK_ENTRY(widg->entry_product)), "-",
                                      gtk_entry_get_text(GTK_ENTRY(widg->entry_version)), "/", NULL);

        if (direntry) {
            if (widg->debug)
                g_printf("direntry: %s\n", direntry);

            if (g_strcasecmp(direntry, textinlist) == 0)
                ok = TRUE;

            g_free(direntry);
        }

        g_free(textinlist);
    }

    return ok;
}

gboolean validate_description_text(struct _allwidgets * widg)
{
    gboolean ok = FALSE;

    GtkTextBuffer *long_text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widg->entry_long));

    if (long_text) {

        GtkTextIter start, end, crtline_end;

        gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(long_text), &start, 0);

        gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(long_text), &end, -1);

        crtline_end = start;

        if (!gtk_text_iter_forward_line(&crtline_end))
            crtline_end = end;

        gchar *tmptext = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(long_text),
                                                  &start, &crtline_end, FALSE);

        if (tmptext) {
            if (strlen(tmptext) > 0)
                ok = TRUE;

            g_free(tmptext);
        }
    }

    return ok;
}

gboolean clear_timer(gpointer data)
{
    struct _allwidgets *widg = data;
    if (widg->timer != 0) {
        g_source_remove(widg->timer);
        widg->timer = 0;
    }
    return FALSE;
}

gboolean operation_finished(gpointer data)
{
    struct _allwidgets *widg = data;

    widg->num_op = 0;

    gnome_druid_set_buttons_sensitive(GNOME_DRUID(widg->druid), TRUE, TRUE, TRUE, FALSE);

    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(widg->progress_check), 1.0);

    gtk_label_set_text(GTK_LABEL(widg->label_dir), "Finished");

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_ready));

    return FALSE;
}

gboolean operation_status_update(gpointer data)
{
    struct _allwidgets *widg = data;

    gdouble frac;
    gchar *step = NULL;

    widg->current_op++;

    if (widg->num_op > 0) {
        frac = widg->current_op / (gdouble) widg->num_op;

        step = g_strdup_printf("Step %d/%d", (widg->current_op + 1), widg->num_op);

        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(widg->progress_check), frac);
    }

    if (step != NULL) {
        gtk_label_set_text(GTK_LABEL(widg->label_dir), step);
        g_free(step);
    }
    return TRUE;
}

gboolean operation_call_first(gpointer data)
{
    struct _allwidgets *widg = data;

    if (do_next_operation(data) == TRUE)
        g_idle_add(operation_finished, data);

    return FALSE;
}

gboolean do_next_operation(gpointer data)
{
    struct _allwidgets *widg = data;
    GList *tmp;
    gboolean fresult;
    gboolean(*func) (gpointer);
    gdouble progress_status;

    gboolean stop = (widg->operation_list == NULL);

    while (stop == FALSE) {
        tmp = widg->operation_list;

        func = tmp->data;

        fresult = func(data);

        if (fresult == TRUE) {
            operation_status_update(data);

            widg->operation_list = g_list_remove(widg->operation_list, func);

            stop = (widg->operation_list == NULL);
        } else {
            widg->num_op = 0;
            if (widg->debug)
                g_printf("failed\n");
            gtk_label_set_text(GTK_LABEL(widg->label_dir), "Failed");
            stop = TRUE;
        }
    }

    if (widg->num_op > 0)
        stop = TRUE;
    else
        stop = FALSE;

    return stop;
}

gboolean execute_command(gchar * cmd)
{
    gboolean rval = FALSE;

    int i = system(cmd);

    g_free(cmd);

    if (i == 0)
        rval = TRUE;

    return rval;
}

gboolean prebuild_dhmake(gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gboolean ok = FALSE;
    if (widg->debug)
        g_printf("dhmake\n");
    gchar *cmd = call_custom_dhmake(widg);

    if (cmd != NULL) {
        if (widg->debug)
            g_printf("cmd: %s\n", cmd);

        ok = execute_command(cmd);
    }

    if (ok == TRUE) {
        const gchar *to = gtk_entry_get_text(GTK_ENTRY(widg->entry_dir));

        cmd = g_strconcat("mv ", to, "/debian/control ", to, "/debian/control_original", NULL);

        if (cmd != NULL) {
            if (widg->debug)
                g_printf("cmd: %s\n", cmd);
            ok = execute_command(cmd);
        } else
            ok = FALSE;
    }

    return ok;

}

gboolean prebuild_check_directories(gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gboolean result = FALSE;

    if (widg->debug)
        g_printf("check directories\n");
    gchar *from = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widg->file_chooser));

    if (from != NULL) {

        const gchar *to = gtk_entry_get_text(GTK_ENTRY(widg->entry_dir));

        gchar *cmd = copy_from_to(widg, from, to);

        g_free(from);

        if (cmd) {
            int retval = system(cmd);

            if (widg->debug)
                g_printf("cmd: %s [%d]\n", cmd, retval);

            if (retval == 0)
                result = TRUE;

            g_free(cmd);
        }
    }

    return result;
}

gboolean prebuild_check_files(gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    if (widg->debug)
        g_printf("check files\n");
    return TRUE;
}

gboolean build_retry(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_ready));

    return TRUE;
}

gboolean clear_oplist(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    if (widg->operation_list) {
        g_list_free(widg->operation_list);
        widg->operation_list = NULL;

        widg->num_op = 0;

    }

    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(widg->progress_check), 0.0);

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_space));

    return TRUE;
}

gboolean dialog_confirm_empty(struct _allwidgets * widg, const gchar * to)
{
    gboolean retval = FALSE;

    GtkWidget *dialog;

    dialog = gtk_message_dialog_new(GTK_WINDOW(widg->window),
                                    GTK_DIALOG_MODAL,
                                    GTK_MESSAGE_QUESTION,
                                    GTK_BUTTONS_OK_CANCEL,
                                    "The folder %s already exists.\n\nAre you sure you want to delete it?",
                                    to);

    if (gtk_dialog_run(GTK_DIALOG(dialog))
        == GTK_RESPONSE_OK) {
        if (delete_projectdir(widg, to) == 0)
            retval = TRUE;
    }

    gtk_widget_destroy(dialog);

    return retval;
}

void show_dialog_message(GtkWidget * window, GtkMessageType message_type, GtkButtonsType button, gchar * text)
{
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new(GTK_WINDOW(window), GTK_DIALOG_MODAL, message_type, button, text);

    gtk_dialog_run(GTK_DIALOG(dialog));

    gtk_widget_destroy(dialog);

}

gboolean check_timer(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gboolean wait = FALSE;

    if (widg->timer != 0) {
        show_dialog_message(widg->window, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, FILELISTING_PROCESS_RUNNING);

        wait = TRUE;
    }

    return wait;
}

gboolean setdir(GnomeDruidPage * druidpage, GtkWidget * widget, gpointer user_data)
{
    struct _allwidgets *widg = user_data;

    gchar *tmpdir = NULL;
    gchar *projectdir = NULL;

    gboolean dirok = TRUE, exist = FALSE;

    const gchar *version = gtk_entry_get_text(GTK_ENTRY(widg->entry_version));

    const gchar *path = g_getenv("HOME");

    if (widg->timer != 0) {
        show_dialog_message(widg->window,
                            GTK_MESSAGE_WARNING, GTK_BUTTONS_CANCEL, FILELISTING_PROCESS_RUNNING);
    } else if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder))
               && !validate_version(widg)) {
        show_dialog_message(widg->window,
                            GTK_MESSAGE_ERROR,
                            GTK_BUTTONS_CANCEL,
                            "Unable to detect the top source folder in the archive. This should be something like <name>-<version>.");
    } else if (strlen(gtk_entry_get_text(GTK_ENTRY(widg->entry_short))) > 0
               && validate_description_text(widg)) {

        if (path) {
            tmpdir = g_strconcat(path, "/.giftwrap", NULL);

            if (tmpdir) {
                create_tmpdir(widg, tmpdir);

                g_free(tmpdir);
                tmpdir = NULL;
            }

            tmpdir = g_strconcat(path, "/.giftwrap/",
                                 gtk_entry_get_text(GTK_ENTRY(widg->entry_product)), NULL);
        }

        if (tmpdir) {
            if (strlen(version) > 0) {
                projectdir = g_strconcat(tmpdir, "/",
                                         gtk_entry_get_text(GTK_ENTRY(widg->entry_product)),
                                         "-", version, NULL);
            } else
                projectdir = g_strconcat(tmpdir, "/",
                                         gtk_entry_get_text(GTK_ENTRY(widg->entry_product)), NULL);

            if (projectdir) {
                gtk_entry_set_text(GTK_ENTRY(widg->entry_dir), projectdir);

                if (g_file_test(tmpdir, G_FILE_TEST_EXISTS))
                    dirok = dialog_confirm_empty(widg, tmpdir);

                if (dirok == TRUE) {
                    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder))) {

                        if (widg->debug)
                            g_print("mkdir %s\n", tmpdir);

                        g_mkdir_with_parents(tmpdir, S_IRWXU | S_IRWXG | S_IRWXG);
                    } else {
                        if (widg->debug)
                            g_print("mkdir %s\n", projectdir);

                        g_mkdir_with_parents(projectdir, S_IRWXU | S_IRWXG | S_IRWXG);
                    }
                }

                g_free(projectdir);
                projectdir = NULL;
            }

            g_free(tmpdir);
            tmpdir = NULL;
        }

        if (dirok == TRUE) {
            gtk_label_set_text(GTK_LABEL(widg->label_dir), "Preparing operation");

            gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_confirm));

            gnome_druid_set_buttons_sensitive(GNOME_DRUID(widg->druid), TRUE, FALSE, TRUE, FALSE);

            widg->operation_list = g_list_prepend(widg->operation_list, prebuild_check_directories);

            widg->operation_list = g_list_prepend(widg->operation_list, prebuild_check_files);

            widg->operation_list = g_list_prepend(widg->operation_list, prebuild_dhmake);

            widg->operation_list = g_list_prepend(widg->operation_list, prebuild_gettext_from_description);

            widg->num_op = g_list_length(widg->operation_list);

            widg->operation_list = g_list_reverse(widg->operation_list);

            widg->current_op = 0;

            g_idle_add(operation_call_first, user_data);
        }

    }
    return TRUE;
}

void build_package(struct _allwidgets *widg, gchar * rootcmd)
{
    gchar *cmd = g_strconcat("cd ",
                             gtk_entry_get_text(GTK_ENTRY(widg->entry_dir)),
                             " && dpkg-buildpackage -r",
                             rootcmd, " 2>&1 | cat > ",
                             gtk_entry_get_text(GTK_ENTRY(widg->entry_dir)),
                             "/giftwrap-build.log", NULL);

    if (cmd) {
        widg->timer = g_timeout_add(100, update_status, widg->progress_build);

        if (widg->cmd)
            g_free(widg->cmd);
        widg->cmd = cmd;

        widg->aftercmd = package_done;

        g_thread_create(action_do, widg, FALSE, NULL);
    }
}

gchar *get_combo_string(GtkWidget * w)
{
    GtkComboBox *combo;
    GtkTreeModel *model;
    GtkTreeIter iter;
    gchar *text;

    combo = GTK_COMBO_BOX(w);
    model = gtk_combo_box_get_model(combo);
    gtk_combo_box_get_active_iter(combo, &iter);
    gtk_tree_model_get(model, &iter, 0, &text, -1);

    return text;
}

gboolean do_prepbuild(struct _allwidgets * widg, const gchar * long_desc, const gchar * short_desc)
{
    gchar *cmd = NULL, *path = NULL, *content = NULL;

    gboolean ok = FALSE;

    gchar *section = get_combo_string(widg->combo_depsection);

    path = g_strconcat(gtk_entry_get_text(GTK_ENTRY(widg->entry_dir)), "/debian/", NULL);

    if (path) {
        if (g_file_test(path, G_FILE_TEST_EXISTS)) {
            cmd = g_strconcat("cd ", path, " && rm *.[eE][xX]", NULL);

            if (cmd != NULL) {
                if (widg->debug)
                    g_print("%s\n", cmd);

                ok = execute_command(cmd);

                cmd = NULL;
            }

        }

        if (widg->debug)
            g_printf("%s\n", long_desc);

        content = g_strconcat("Source: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_product)),
                              "\nSection: ",
                              section,
                              "\nPriority: optional\nMaintainer: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_name)),
                              " <", gtk_entry_get_text(GTK_ENTRY(widg->entry_email)),
                              ">\nBuild-Depends: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depbuild)),
                              "\nStandards-Version: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_deppolicy)),
                              "\n\nPackage: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_deppkgname)),
                              "\nArchitecture: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_pkgarch)),
                              "\nDepends: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depinstall)),
                              "\nRecommends: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_deprecommends)),
                              "\nSuggests: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depsuggests)),
                              "\nPre-Depends: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_deppredepends)),
                              "\nConflicts: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depconflicts)),
                              "\nProvides: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depprovides)),
                              "\nReplaces: ",
                              gtk_entry_get_text(GTK_ENTRY(widg->entry_depreplaces)),
                              "\nDescription: ", short_desc, "\n ", long_desc, "\n", NULL);

        if (content) {
            cmd = g_strconcat(path, "control", NULL);

            if (cmd) {
                ok = g_file_set_contents(cmd, content, -1, NULL);

                g_free(cmd);
                cmd = NULL;
            }

            g_free(content);
            content = NULL;
        }

        g_free(path);
        path = NULL;
    }

    if (section)
        g_free(section);

    return ok;

}

gchar *description_control_file(struct _allwidgets * widg,
                                const GtkTextBuffer * long_text, const gchar * short_desc)
{
    gint i, max = gtk_text_buffer_get_line_count(GTK_TEXT_BUFFER(long_text));

    GtkTextIter liter_s, liter_e;

    gchar *line_desc, *long_desc = NULL, *tmp_desc = NULL;

    for (i = 0; i < max; i++) {
        gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(long_text), &liter_s, i);

        liter_e = liter_s;

        gtk_text_iter_forward_line(&liter_e);

        line_desc = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(long_text), &liter_s, &liter_e, FALSE);

        if (line_desc) {
            if (strlen(line_desc) > 0) {
                line_desc = g_strchomp(line_desc);

                gtk_text_iter_backward_char(&liter_e);

                if ('\n' == gtk_text_iter_get_char(&liter_e)) {
                    if (strlen(line_desc) == 0) {
                        g_free(line_desc);
                        line_desc = g_strdup(".\n");
                    } else {
                        tmp_desc = line_desc;

                        line_desc = g_strconcat(tmp_desc, "\n", NULL);

                        g_free(tmp_desc);
                        tmp_desc = NULL;
                    }

                }

                if (line_desc) {
                    if (long_desc)
                        long_desc = g_strconcat(long_desc, " ", line_desc, NULL);
                    else
                        long_desc = g_strconcat(" ", line_desc, NULL);
                }

            }

            g_free(line_desc);
        }
    }

    return long_desc;
}

gboolean prebuild_gettext_from_description(gpointer user_data)
{
    struct _allwidgets *widg = user_data;
    gboolean build = FALSE;

    if (widg->debug)
        g_printf("replace desc\n");

    GtkTextBuffer *long_text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widg->entry_long));

    if (long_text) {
        const gchar *short_desc = gtk_entry_get_text(GTK_ENTRY(widg->entry_short));

        gchar *long_desc = description_control_file(widg,
                                                    long_text, short_desc);

        if (long_desc) {
            build = do_prepbuild(widg, long_desc, short_desc);

            g_free(long_desc);
        }
    }

    if (widg->debug) {
        if (build == TRUE)
            g_printf("build=TRUE\n");
        else
            g_printf("build=FALSE\n");
    }

    return build;
}

void switch_filechooser_type(struct _allwidgets *widg)
{
    if (gtk_combo_box_get_active(GTK_COMBO_BOX(widg->combo_type)) == 1) {

        gtk_file_chooser_set_action(GTK_FILE_CHOOSER(widg->file_chooser), GTK_FILE_CHOOSER_ACTION_OPEN);

    } else {
        gtk_file_chooser_set_action(GTK_FILE_CHOOSER(widg->file_chooser),
                                    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
    }

}

void set_filechooser_type(GtkComboBox * widget, gpointer data)
{
    struct _allwidgets *widg = data;

    switch_filechooser_type(widg);
}
