/*   
    (C) 2009 Vadim Peretokin
    
    (C) 2007, 2008 Gianni Moschini

    This file is part of GiftWrap.

    GiftWrap is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    GiftWrap is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GiftWrap.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <libgen.h>
#include <libgnomeui/libgnomeui.h>
#include "def.h"
#include "action.h"
#include "callback.h"
#include "config.h"

void show_last_page(struct _allwidgets *widg, gboolean done)
{
    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_end));

    gnome_druid_set_buttons_sensitive(GNOME_DRUID(widg->druid), !done, FALSE, !done, FALSE);

    if (done) {
        gnome_druid_set_show_finish(GNOME_DRUID(widg->druid), TRUE);
    }

    if (!gtk_window_has_toplevel_focus(GTK_WINDOW(widg->window)))
        gtk_window_set_urgency_hint(GTK_WINDOW(widg->window), TRUE);

}

void clear_commandtimer(struct _allwidgets *widg)
{
    if (widg->aftercmd)
        widg->aftercmd = NULL;

    if (widg->timer != 0) {
        g_source_remove(widg->timer);
        widg->timer = 0;
    }

    if (widg->cmd) {
        g_free(widg->cmd);
        widg->cmd = NULL;
    }

}

gboolean movecontrol_done(gpointer data)
{
    struct _allwidgets *widg = data;

    clear_commandtimer(widg);

    gnome_druid_set_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_desc));

    gnome_druid_set_buttons_sensitive(GNOME_DRUID(widg->druid), FALSE, TRUE, TRUE, FALSE);

    return FALSE;
}

gboolean dhmake_done(gpointer data)
{
    struct _allwidgets *widg = data;

    const gchar *to = gtk_entry_get_text(GTK_ENTRY(widg->entry_dir));;

    if (widg->aftercmd)
        widg->aftercmd = NULL;

    if (widg->cmd)
        g_free(widg->cmd);

    widg->cmd = g_strconcat("mv ", to, "/debian/control ", to, "/debian/control_original", NULL);

    if (widg->cmd) {
        widg->aftercmd = movecontrol_done;

        g_thread_create(action_do, widg, FALSE, NULL);
    }

    return FALSE;
}

gchar *call_custom_dhmake(struct _allwidgets * widg)
{
    const gchar *to, *email, *name;
    gchar *cmd = NULL;
    gchar *redirect = NULL;
    gchar *textlicense = NULL;
    gchar *license = get_combo_string(widg->combo_license);

    if (license) {
        if (g_strcasecmp(license, "other") != 0)
            textlicense = g_strconcat(" -c ", license, NULL);
        else
            textlicense = g_strdup(" ");

        g_free(license);
        license = NULL;
    }

    if (textlicense) {
        if (!widg->debug)
            redirect = g_strdup(" </dev/null");

        to = gtk_entry_get_text(GTK_ENTRY(widg->entry_dir));

        email = gtk_entry_get_text(GTK_ENTRY(widg->entry_email));

        name = gtk_entry_get_text(GTK_ENTRY(widg->entry_name));

        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widg->checkfolder)))

            cmd = g_strconcat("cd ", to,
                              " && DEBEMAIL=\"", email,
                              "\" DEBFULLNAME=\"", name, "\" ",
                              CUSTOM_DHMAKE_PATH, "/", "dh_make",
                              " --single ", textlicense, " --createorig ", redirect, NULL);
        else {
            gchar *basename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widg->file_chooser));

            if (basename) {
                gchar *filename = g_path_get_basename(basename);

                g_free(basename);

                if (filename) {
                    cmd = g_strconcat("cd ", to,
                                      " && DEBEMAIL=\"", email,
                                      "\" DEBFULLNAME=\"", name, "\" ",
                                      CUSTOM_DHMAKE_PATH, "/", "/dh_make",
                                      " --single ", textlicense, " -f ../", filename, redirect, NULL);

                    g_free(filename);
                }
            }
        }

        g_free(textlicense);

        if (redirect)
            g_free(redirect);

    }

    return cmd;
}

gchar *check_debianfiles(struct _allwidgets * widg, const gchar * files)
{
    gchar *cmd = NULL, *output = NULL;

    cmd = g_strconcat("cut -d' ' -f1 ", files, NULL);

    if (cmd) {
        if (widg->debug)
            g_print("%s\n", cmd);

        g_spawn_command_line_sync(cmd, &output, NULL, NULL, NULL);

        g_free(cmd);
    }

    return output;
}

gboolean debpackage_exists(struct _allwidgets * widg, const gchar * output)
{
    gchar *debfile = NULL;

    gboolean done = FALSE;

    gchar *dirname = g_path_get_dirname(widg->cmd);

    if (widg->debug)
        g_print("%s\n", output);

    if (dirname) {
        debfile = g_strconcat(dirname, "/", output, NULL);

        if (debfile) {
            if (widg->debug)
                g_print("%s\n", debfile);

            done = g_file_test(debfile, G_FILE_TEST_EXISTS);

            g_free(debfile);
        }

        g_free(dirname);
    }

    return done;
}

void set_deb_entry(struct _allwidgets *widg, gchar * debname)
{
    gtk_label_set_markup(GTK_LABEL(widg->label_end), "<b>Package created successfully.</b>");

    gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_end), "Success");

    gtk_entry_set_text(GTK_ENTRY(widg->entry_end), debname);

    gtk_entry_set_editable(GTK_ENTRY(widg->entry_end), FALSE);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widg->check_end), TRUE);

    gtk_widget_show(widg->entry_end);
    gtk_widget_show(widg->check_end);

}

gboolean check_package_file(struct _allwidgets *widg)
{
    gboolean done = FALSE;

    gchar *files = g_strconcat(widg->cmd,
                               "/debian/files", NULL);

    gchar *debfilename = NULL;

    if (files) {
        if (g_file_test(files, G_FILE_TEST_EXISTS))
            debfilename = check_debianfiles(widg, files);

        g_free(files);
    }

    if (debfilename) {
        if (strlen(debfilename) > 0)
            debfilename[strlen(debfilename) - 1] = '\0';

        done = debpackage_exists(widg, debfilename);

        if (done)
            set_deb_entry(widg, debfilename);

        g_free(debfilename);
    }

    return done;
}

void show_error_view(struct _allwidgets *widg, const gchar * content)
{
    GtkTextBuffer *text;

    text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widg->text_end));

    gtk_text_buffer_set_text(GTK_TEXT_BUFFER(text), content, -1);

    gtk_widget_show(widg->scrolled_end);
    gtk_widget_hide(widg->entry_end);
    gtk_widget_hide(widg->check_end);

}

void show_build_log(struct _allwidgets *widg)
{
    gchar *path = g_strconcat(gtk_entry_get_text(GTK_ENTRY(widg->entry_dir)),
                              "/giftwrap-build.log", NULL);

    if (path) {
        gchar *content = NULL;

        if (g_file_test(path, G_FILE_TEST_EXISTS)) {
            if (g_file_get_contents(path, &content, NULL, NULL)) {

                if (content)
                    show_error_view(widg, content);
            }

            if (content)
                g_free(content);

        } else {
            show_error_view(widg,
                            "Unable to show debug file. Something went wrong before we can even start the build process.");
        }

        g_free(path);
    }

}

gboolean package_done(gpointer data)
{
    struct _allwidgets *widg = data;

    gboolean done = FALSE;

    clear_commandtimer(widg);

    if (widg->cmd)
        g_free(widg->cmd);

    widg->cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widg->entry_dir)));

    if (widg->cmd)
        done = check_package_file(widg);

    if (!done) {
        show_build_log(widg);

        if (widg->cmd) {
            g_free(widg->cmd);
            widg->cmd = NULL;
        }
    } else {
        gtk_widget_hide(widg->scrolled_end);
    }

    show_last_page(widg, done);

    return FALSE;
}
