/*   
    (C) 2009 Vadim Peretokin
    
    (C) 2007, 2008 Gianni Moschini

    This file is part of GiftWrap.

    GiftWrap is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    GiftWrap is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GiftWrap.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <libgnomeui/libgnomeui.h>
#include <glib/gi18n.h>

#include "def.h"
#include "callback.h"
#include "config.h"
#include "sections_trans.h"
#include "icons.h"

void interface_attach_entrylabel_totable(GtkWidget * table, GtkWidget * label, GtkWidget * entry, int row)
{
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);

    gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
}

void interface_create_labelentry(GtkWidget ** label,
                                 const gchar * text_label, GtkWidget ** entry, const gchar * text_entry)
{
    *label = gtk_label_new(text_label);

    gtk_misc_set_alignment(GTK_MISC(*label), 0, 0.5);

    *entry = gtk_entry_new();

    if (text_entry)
        gtk_entry_set_text(GTK_ENTRY(*entry), text_entry);
}

void interface_create_combo(GtkWidget ** label,
                            const gchar * text_label,
                            GtkWidget ** combo, const gchar * defaulttext, char *table[][2])
{
    GtkListStore *model;
    GtkTreeIter init;
    GtkCellRenderer *renderer;
    gint i;

    *label = gtk_label_new(text_label);

    gtk_misc_set_alignment(GTK_MISC(*label), 0, 0.5);

    model = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
    for (i = 0; table[i][0] != NULL; i++) {
        GtkTreeIter iter;
        gtk_list_store_append(model, &iter);
        gtk_list_store_set(model, &iter, 0, table[i][0], 1, table[i][1], -1);
        if (strcmp(table[i][0], defaulttext) == 0)
            init = iter;
    }
    *combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(model));
    renderer = gtk_cell_renderer_text_new();
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(*combo), renderer, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(*combo), renderer, "text", 1);
    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(*combo), &init);
}

void build_interface(int argc, char *argv[])
{

    struct _allwidgets *widg = (struct _allwidgets *)
        g_malloc(sizeof(struct _allwidgets));

    GtkFileFilter *filter, *nofilter;
    GList *icons = NULL;

    if (widg) {
        widg->timer = 0;
        widg->childpid = -1;
        widg->num_op = 0;
        widg->current_op = 0;
        widg->cmd = NULL;
        widg->aftercmd = NULL;
        widg->workingdir = NULL;
        widg->operation_list = NULL;
        widg->output = NULL;
        widg->mimefilters = NULL;

        if (argc > 1 && g_ascii_strcasecmp(argv[1], "--debug") == 0)
            widg->debug = TRUE;
        else
            widg->debug = FALSE;

        widg->mimefilters = g_slist_prepend(widg->mimefilters, g_strdup("application/x-bzip-compressed-tar"));

        widg->mimefilters = g_slist_prepend(widg->mimefilters, g_strdup("application/x-compressed-tar"));

        widg->mimefilters = g_slist_prepend(widg->mimefilters, g_strdup("application/x-tar"));

        widg->druid = gnome_druid_new_with_window(PACKAGE_NAME, NULL, TRUE, &widg->window);

        /* add the program logos to the window */
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_16px, FALSE, NULL));
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_22px, FALSE, NULL));
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_32px, FALSE, NULL));
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_48px, FALSE, NULL));
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_64px, FALSE, NULL));
        icons = g_list_prepend(icons, gdk_pixbuf_new_from_inline (-1, main_logo_128px, FALSE, NULL));

        gtk_window_set_icon_list(GTK_WINDOW(widg->window), icons);

        g_list_foreach(icons, (GFunc)&g_object_unref, NULL);
        g_list_free(icons);

        /*gnome_druid_set_buttons_sensitive(
           GNOME_DRUID(widg->druid), FALSE, TRUE, TRUE, FALSE); */

        widg->page_first = gnome_druid_page_edge_new(GNOME_EDGE_START);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_first));

        gnome_druid_page_edge_set_title(GNOME_DRUID_PAGE_EDGE(widg->page_first), "GiftWrap " VERSION);

        gnome_druid_page_edge_set_text(GNOME_DRUID_PAGE_EDGE(widg->page_first),
                                       "This wizard will help you to build your package from sources. \n\nEach software needs a different set of libraries and headers to be installed to be able to build them. Please install them first according to the README file provided with the software - otherwise, you can do this later when the configuration part fails.\n\nWhen you are ready you can continue and select the project archive or top source code folder.");

        /*
         * Page 1 =====================
         */
        widg->page_select_folder = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_select_folder),
                                            "Select the application archive");

        widg->label = gtk_label_new(NULL);

        gtk_misc_set_alignment(GTK_MISC(widg->label), 0, 0.5);

        gtk_label_set_line_wrap(GTK_LABEL(widg->label), TRUE);

        gtk_label_set_markup(GTK_LABEL(widg->label),
                             "Please select the source folder or archive of the application you want to build.");

        widg->combo_type = gtk_combo_box_new_text();

        gtk_combo_box_append_text(GTK_COMBO_BOX(widg->combo_type), "directory");

        gtk_combo_box_append_text(GTK_COMBO_BOX(widg->combo_type), "archive");

        gtk_combo_box_set_active(GTK_COMBO_BOX(widg->combo_type), 1);

        widg->hbox = gtk_hbox_new(FALSE, 5);

        widg->label_type = gtk_label_new("Type: ");

        gtk_misc_set_alignment(GTK_MISC(widg->label_type), 1, 0.5);

        gtk_box_pack_start(GTK_BOX(widg->hbox), widg->label_type, TRUE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(widg->hbox), widg->combo_type, FALSE, FALSE, 0);

        widg->file_chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN);

        widg->checkfolder = gtk_check_button_new_with_label("Select the top source folder");
        
        gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(widg->file_chooser), widg->checkfolder);

        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(widg->file_chooser), g_get_home_dir());

        filter = gtk_file_filter_new();

        if (filter) {
            GSList *tmp;

            for (tmp = widg->mimefilters; tmp; tmp = tmp->next) {
                gtk_file_filter_add_mime_type(GTK_FILE_FILTER(filter), (gchar *) tmp->data);
            }

            gtk_file_filter_set_name(GTK_FILE_FILTER(filter), "Archives");

            gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widg->file_chooser), filter);
        }

        nofilter = gtk_file_filter_new();

        if (nofilter) {
            gtk_file_filter_set_name(GTK_FILE_FILTER(nofilter), "All files");

            gtk_file_filter_add_pattern(GTK_FILE_FILTER(nofilter), "*");

            gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widg->file_chooser), nofilter);
        }

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_select_folder)->vbox),
                           widg->file_chooser, TRUE, TRUE, 2);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_select_folder));

        /*
         * Page 2 =====================
         */
        widg->page_space = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_space), "Package meta-data");

        widg->vbox_sp1 = gtk_vbox_new(FALSE, 0);
        widg->vbox_sp2 = gtk_vbox_new(FALSE, 0);
        widg->vbox_sp3 = gtk_vbox_new(FALSE, 0);

        gtk_container_set_border_width(GTK_CONTAINER(widg->vbox_sp1), 10);

        gtk_container_set_border_width(GTK_CONTAINER(widg->vbox_sp2), 10);

        gtk_container_set_border_width(GTK_CONTAINER(widg->vbox_sp3), 10);

        widg->label_email = gtk_label_new("Email:");
        gtk_misc_set_alignment(GTK_MISC(widg->label_email), 0, 0.5);
        widg->label_soft = gtk_label_new("Name:");
        gtk_misc_set_alignment(GTK_MISC(widg->label_soft), 0, 0.5);
        widg->label_version = gtk_label_new("Version:");
        gtk_misc_set_alignment(GTK_MISC(widg->label_version), 0, 0.5);
        widg->label_name = gtk_label_new("Packager:");
        gtk_misc_set_alignment(GTK_MISC(widg->label_name), 0, 0.5);

        widg->entry_email = gtk_entry_new();
        widg->entry_name = gtk_entry_new();
        widg->entry_version = gtk_entry_new();
        gtk_widget_set_size_request(widg->entry_version, 140, -1);
        widg->entry_product = gtk_entry_new();

        interface_create_combo(&widg->label_license, "License:", &widg->combo_license, "gpl", licensetable);

        widg->label_maint = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_maint), "<b>Maintainer</b>");

        gtk_misc_set_alignment(GTK_MISC(widg->label_maint), 0, 0);

        widg->label_text = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_text), "<b>Software</b>");

        gtk_misc_set_alignment(GTK_MISC(widg->label_text), 0, 0.5);

        gtk_label_set_line_wrap(GTK_LABEL(widg->label_text), TRUE);

        widg->table_info = gtk_table_new(8, 5, FALSE);

        gtk_table_set_col_spacings(GTK_TABLE(widg->table_info), 5);
        gtk_table_set_row_spacings(GTK_TABLE(widg->table_info), 2);

        /* Line1 */
        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_maint, 0, 5, 0, 1, GTK_FILL, 0, 0, 0);

        /* Line2 */
        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_name, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->entry_name, 1, 5, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);

        /* Line3 */
        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_email, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->entry_email, 1, 5, 2, 3, GTK_FILL | GTK_EXPAND, 0, 0, 0);

        gtk_table_set_row_spacing(GTK_TABLE(widg->table_info), 2, 20);

        /* Line 4 */
        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_text, 0, 5, 3, 4, GTK_FILL, 0, 0, 0);

        /* Line5 */
        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_soft, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->entry_product, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, 0, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_version, 3, 4, 4, 5, GTK_FILL, 0, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info), widg->entry_version, 4, 5, 4, 5, GTK_FILL, 0, 0, 0);

        /* Line6 */

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->label_license, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->combo_license, 1, 5, 5, 6, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

        /* Line7 */
        widg->label_short = gtk_label_new("Synopsis:");

        widg->entry_short = gtk_entry_new();

        gtk_widget_set_tooltip_text(widg->label_short,
                                    "Required: short name of the program - describe what it is about in a single sentence.");
        gtk_widget_set_tooltip_text(widg->entry_short,
                                    "Required: short name of the program - describe what it is about in a single sentence.");

        gtk_misc_set_alignment(GTK_MISC(widg->label_short), 0, 0.5);

        gtk_entry_set_max_length(GTK_ENTRY(widg->entry_short), 60);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->label_short, 0, 1, 6, 7, GTK_FILL, GTK_FILL, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->entry_short, 1, 5, 6, 7, GTK_FILL | GTK_EXPAND, 0, 0, 0);

        /* Line8 */
        gtk_table_set_row_spacing(GTK_TABLE(widg->table_info), 5, 3);

        widg->label_long = gtk_label_new("Description:");

        gtk_misc_set_alignment(GTK_MISC(widg->label_long), 0, 0);

        widg->entry_long = gtk_text_view_new();

        gtk_widget_set_tooltip_text(widg->label_long,
                                    "Required: the full description of the program - describe what it's about, what it does, and mention it's homepage.");
        gtk_widget_set_tooltip_text(widg->entry_long,
                                    "Required: the full description of the program - describe what it's about, what it does, and mention it's homepage.");

        widg->scrolled = gtk_scrolled_window_new(NULL, NULL);

        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widg->scrolled),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(widg->scrolled), GTK_SHADOW_ETCHED_IN);

        gtk_container_add(GTK_CONTAINER(widg->scrolled), widg->entry_long);

        gtk_table_attach(GTK_TABLE(widg->table_info), widg->label_long, 0, 1, 7, 8, GTK_FILL, GTK_FILL, 0, 0);

        gtk_table_attach(GTK_TABLE(widg->table_info),
                         widg->scrolled, 1, 5, 7, 8, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);

        /* Table dep */
        interface_create_labelentry(&widg->label_depinstall,
                                    "Depends:",
                                    &widg->entry_depinstall, "${shlibs:Depends}, ${misc:Depends}");

        interface_create_labelentry(&widg->label_depbuild,
                                    "Build-Depends:",
                                    &widg->entry_depbuild, "debhelper (>= 4.0.0), autotools-dev");

        interface_create_combo(&widg->label_depsection,
                               "Section:", &widg->combo_depsection, "unknown", transtable);

        interface_create_labelentry(&widg->label_deppolicy,
                                    "Debian policy:", &widg->entry_deppolicy, "3.6.2");

        interface_create_labelentry(&widg->label_deppkgname, "Package:", &widg->entry_deppkgname, NULL);

        interface_create_labelentry(&widg->label_deprecommends,
                                    "Recommends:", &widg->entry_deprecommends, NULL);

        interface_create_labelentry(&widg->label_depsuggests, "Suggests:", &widg->entry_depsuggests, NULL);

        interface_create_labelentry(&widg->label_deppredepends,
                                    "Pre-Depends:", &widg->entry_deppredepends, NULL);

        interface_create_labelentry(&widg->label_depconflicts, "Conflicts:", &widg->entry_depconflicts, NULL);

        interface_create_labelentry(&widg->label_depprovides, "Provides:", &widg->entry_depprovides, NULL);

        interface_create_labelentry(&widg->label_depreplaces, "Replaces:", &widg->entry_depreplaces, NULL);

        interface_create_labelentry(&widg->label_pkgarch, "Architecture:", &widg->entry_pkgarch, "any");

        widg->table_depend = gtk_table_new(12, 2, FALSE);
        gtk_table_set_col_spacings(GTK_TABLE(widg->table_depend), 5);
        gtk_table_set_row_spacings(GTK_TABLE(widg->table_depend), 2);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depbuild, widg->entry_depbuild, 0);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depsection, widg->combo_depsection, 1);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_deppolicy, widg->entry_deppolicy, 2);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_deppkgname, widg->entry_deppkgname, 3);

        interface_attach_entrylabel_totable(widg->table_depend, widg->label_pkgarch, widg->entry_pkgarch, 4);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depinstall, widg->entry_depinstall, 5);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_deprecommends, widg->entry_deprecommends, 6);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depsuggests, widg->entry_depsuggests, 7);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_deppredepends, widg->entry_deppredepends, 8);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depconflicts, widg->entry_depconflicts, 9);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depprovides, widg->entry_depprovides, 10);

        interface_attach_entrylabel_totable(widg->table_depend,
                                            widg->label_depreplaces, widg->entry_depreplaces, 11);

        GtkWidget *label_fortmp = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(label_fortmp), "<b>From: </b>");

        GtkWidget *hbox_tmpdir = gtk_hbox_new(FALSE, 5);

        gtk_box_pack_start(GTK_BOX(hbox_tmpdir), label_fortmp, FALSE, FALSE, 0);

        widg->label_tmpdirectory = gtk_label_new(NULL);

        gtk_label_set_ellipsize(GTK_LABEL(widg->label_tmpdirectory), PANGO_ELLIPSIZE_END);

        gtk_label_set_selectable(GTK_LABEL(widg->label_tmpdirectory), TRUE);

        gtk_box_pack_start(GTK_BOX(hbox_tmpdir), widg->label_tmpdirectory, TRUE, TRUE, 0);

        widg->textview_listfiles = gtk_text_view_new();

        gtk_text_view_set_editable(GTK_TEXT_VIEW(widg->textview_listfiles), FALSE);

        widg->scrolled_listfiles = gtk_scrolled_window_new(NULL, NULL);

        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widg->scrolled_listfiles),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(widg->scrolled_listfiles),
                                            GTK_SHADOW_ETCHED_IN);

        gtk_container_add(GTK_CONTAINER(widg->scrolled_listfiles), widg->textview_listfiles);

        widg->progress_list = gtk_progress_bar_new();

        /* Packaging */

        gtk_box_pack_start(GTK_BOX(widg->vbox_sp1), widg->table_info, TRUE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(widg->vbox_sp2), widg->table_depend, FALSE, TRUE, 0);

        gtk_box_set_spacing(GTK_BOX(widg->vbox_sp3), 10);

        gtk_box_pack_start(GTK_BOX(widg->vbox_sp3), hbox_tmpdir, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(widg->vbox_sp3), widg->progress_list, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(widg->vbox_sp3), widg->scrolled_listfiles, TRUE, TRUE, 0);

        widg->notebook_infos = gtk_notebook_new();

        gtk_notebook_append_page(GTK_NOTEBOOK(widg->notebook_infos),
                                 widg->vbox_sp1, gtk_label_new("General"));

        gtk_notebook_append_page(GTK_NOTEBOOK(widg->notebook_infos),
                                 widg->vbox_sp2, gtk_label_new("Optional"));

        gtk_notebook_append_page(GTK_NOTEBOOK(widg->notebook_infos),
                                 widg->vbox_sp3, gtk_label_new("Files listing"));

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_space)->vbox),
                           widg->notebook_infos, TRUE, TRUE, 0);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_space));

        /*
         * Page 3 =====================
         */
        widg->page_confirm = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_confirm),
                                            "Prebuild checking");

        widg->progress_check = gtk_progress_bar_new();

        widg->label_dir = gtk_label_new(NULL);

        gtk_misc_set_alignment(GTK_MISC(widg->label_dir), 0, 0.5);

        widg->entry_dir = gtk_entry_new();

        widg->label_whatnext = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_whatnext),
                             "<b>Auto-check</b>\n\nThe program does some checks before starting the build process. You will not be able to continue if one of those checks fails.");

        gtk_misc_set_alignment(GTK_MISC(widg->label_whatnext), 0, 0.5);

        gtk_label_set_line_wrap(GTK_LABEL(widg->label_whatnext), TRUE);

        widg->treeview_check = gtk_tree_view_new();

        GtkWidget *scrolled_check = gtk_scrolled_window_new(NULL, NULL);

        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_check),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_check), GTK_SHADOW_ETCHED_IN);

        gtk_container_add(GTK_CONTAINER(scrolled_check), widg->treeview_check);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_confirm)->vbox),
                           widg->label_whatnext, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_confirm)->vbox),
                           widg->label_dir, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_confirm)->vbox),
                           widg->progress_check, FALSE, TRUE, 0);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_confirm));

        /*
         * Page 4 =====================
         */

        widg->page_ready = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_ready), "Ready to build");

        widg->label_ready = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_ready),
                             "<b>Building package</b>\n\nPlease click forward to start the building process.");

        gtk_misc_set_alignment(GTK_MISC(widg->label_ready), 0, 0.5);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_ready)->vbox),
                           widg->label_ready, FALSE, TRUE, 0);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_ready));

        /*
         * Page 5 =====================
         */
        widg->page_desc = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_desc), "Skip");

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_desc));

        /*
         * Page 6 =====================
         */
        widg->page_doing = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_doing), "Building");

        widg->progress_build = gtk_progress_bar_new();

        widg->label_wait = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_wait), "<b>Building package, please wait...</b>");

        gtk_misc_set_alignment(GTK_MISC(widg->label_wait), 0, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_doing)->vbox),
                           widg->label_wait, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_doing)->vbox),
                           widg->progress_build, FALSE, TRUE, 0);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_doing));

        /*
         * Final page  =====================
         */
        widg->page_end = gnome_druid_page_standard_new();

        gnome_druid_page_standard_set_title(GNOME_DRUID_PAGE_STANDARD(widg->page_end), "Build failed");

        widg->check_end = gtk_check_button_new_with_label("Open target folder");

        widg->text_end = gtk_text_view_new();

        gtk_text_view_set_editable(GTK_TEXT_VIEW(widg->text_end), FALSE);

        widg->scrolled_end = gtk_scrolled_window_new(NULL, NULL);

        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widg->scrolled_end),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(widg->scrolled_end), GTK_SHADOW_ETCHED_IN);

        gtk_container_add(GTK_CONTAINER(widg->scrolled_end), widg->text_end);

        /* Default error message */
        widg->label_end = gtk_label_new(NULL);

        gtk_label_set_markup(GTK_LABEL(widg->label_end), "<b>Unable to create package.</b>");

        gtk_misc_set_alignment(GTK_MISC(widg->label_end), 0, 0);

        widg->entry_end = gtk_entry_new();

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_end)->vbox),
                           widg->label_end, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_end)->vbox),
                           widg->scrolled_end, TRUE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_end)->vbox),
                           widg->entry_end, FALSE, TRUE, 0);

        gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(widg->page_end)->vbox),
                           widg->check_end, FALSE, TRUE, 0);

        gnome_druid_append_page(GNOME_DRUID(widg->druid), GNOME_DRUID_PAGE(widg->page_end));

        /* 
         * Events  =====================
         */
        g_signal_connect(G_OBJECT(widg->window), "delete-event", G_CALLBACK(quit), (gpointer) widg);

        g_signal_connect(G_OBJECT(widg->window), "window-state-event", G_CALLBACK(clear_hint), widg);

        g_signal_connect(G_OBJECT(widg->page_confirm), "back", G_CALLBACK(clear_oplist), widg);
        g_signal_connect(G_OBJECT(widg->page_end), "back", G_CALLBACK(build_retry), widg);

        g_signal_connect(G_OBJECT(widg->page_space), "back", G_CALLBACK(check_timer), widg);

        g_signal_connect(G_OBJECT(widg->page_select_folder), "next", G_CALLBACK(check_selected_file), widg);

        g_signal_connect(G_OBJECT(widg->page_space), "next", G_CALLBACK(setdir), widg);

        g_signal_connect(G_OBJECT(widg->page_confirm), "next", G_CALLBACK(switchto_pageready), widg);
        g_signal_connect(G_OBJECT(widg->page_desc), "next", G_CALLBACK(start_build_page), widg);

        g_signal_connect(G_OBJECT(widg->page_ready), "next", G_CALLBACK(start_build_page), widg);

        g_signal_connect(G_OBJECT(widg->page_end), "finish", G_CALLBACK(finish), widg);

        g_signal_connect(G_OBJECT(widg->page_end), "cancel", G_CALLBACK(cancel), widg);
        g_signal_connect(G_OBJECT(widg->page_select_folder), "cancel", G_CALLBACK(cancel), widg);
        g_signal_connect(G_OBJECT(widg->page_space), "cancel", G_CALLBACK(cancel), widg);
        g_signal_connect(G_OBJECT(widg->page_confirm), "cancel", G_CALLBACK(cancel), widg);
        g_signal_connect(G_OBJECT(widg->page_desc), "cancel", G_CALLBACK(cancel), widg);

        g_signal_connect(G_OBJECT(widg->page_first), "cancel", G_CALLBACK(cancel), widg);

        g_signal_connect(G_OBJECT(widg->page_doing), "cancel", G_CALLBACK(cancel), widg);
        g_signal_connect(G_OBJECT(widg->page_ready), "cancel", G_CALLBACK(cancel), widg);

        g_signal_connect(G_OBJECT(widg->file_chooser), "file-activated", G_CALLBACK(fileselection), widg);

        g_signal_connect(G_OBJECT(widg->combo_type),
                         "changed", G_CALLBACK(set_filechooser_type), (gpointer) widg);

        g_signal_connect(G_OBJECT(widg->checkfolder),
                         "toggled", G_CALLBACK(select_currentdir_toggle), (gpointer) widg);

        /*
         * Size and Show =====================
         */
        /*gtk_widget_set_size_request(widg->window, 520, 400); */

        gtk_widget_show_all(widg->window);
    }
}

void show_error_message(const gchar * text)
{
    GtkWidget *dialog = gtk_dialog_new_with_buttons("Installation error.", NULL, 0,
                                                    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);

    if (dialog) {
        GtkWidget *error_text = gtk_label_new(text);

        gtk_misc_set_alignment(GTK_MISC(error_text), 0, 0);

        gtk_widget_set_size_request(error_text, 260, 150);

        gtk_label_set_line_wrap(GTK_LABEL(error_text), TRUE);

        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), error_text);

        gtk_widget_show_all(GTK_DIALOG(dialog)->vbox);

        gtk_dialog_run(GTK_DIALOG(dialog));

        gtk_widget_destroy(dialog);
    }

}

int main(int argc, char *argv[])
{
    if (!g_thread_supported()) {
        g_thread_init(NULL);
        gdk_threads_init();
    }

    bindtextdomain(PACKAGE, GNOMELOCALEDIR);
    bind_textdomain_codeset(PACKAGE, "UTF-8");
    textdomain(PACKAGE);

    gnome_program_init(PACKAGE, PACKAGE_VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);

    if (g_file_test(CUSTOM_DHMAKE_PATH "/dh_make", G_FILE_TEST_EXISTS)) {
        if (g_file_test(CUSTOM_DHMAKE_PATH "/atool", G_FILE_TEST_EXISTS)) {
            build_interface(argc, argv);

            gtk_main();
        } else {
            g_critical("Unable to find atool.");

            g_print("Please check your installation.\n");
            g_print("Error: %s not found.\n", "atool");

            show_error_message
                ("ERROR: unable to find the requires atool script.\n\nPlease install it from your package manager.");
        }
    } else {
        g_critical("Unable to find dh_make script.");

        g_print("Please check your installation.\n");
        g_print("Error: %s not found.\n", CUSTOM_DHMAKE_PATH "/dh_make");

        show_error_message
            ("ERROR: unable to find dhmake script provided with giftwrap.\n\nYou can run this tool from a terminal to get more details.\n\nPlease check your installation.\n");
    }

    return (0);
}
